﻿using UnityEngine;
using System.IO.Ports;
using System.IO;

[RequireComponent(typeof(CharacterController))]

public class FirstPersonController : MonoBehaviour
{
    public int numberOfRays = 8;
    public float angleOfViewInRadian = 1.3f;

    public float servoFramRate = 0.2f;
    public float walkingSpeed = 7.5f;
    public float runningSpeed = 11.5f;
    public float jumpSpeed = 8.0f;
    public float gravity = 20.0f;
    public Camera playerCamera;
    public float lookSpeed = 2.0f;
    public float lookXLimit = 45.0f;

    CharacterController characterController;
    Vector3 moveDirection = Vector3.zero;
    float rotationX = 0;

    [HideInInspector]
    public bool canMove = true;

    SerialPort sp;
    private bool isFalling = false;
    private Quaternion originalRotation;
    private int currentWrongWayIndex = -1;

    [SerializeField]
    private AudioSource wrongWaySound;

    public void wrongWayIndexChanged(int newIndex)
    {
        Debug.Log("new index: " + newIndex);
        if (newIndex <= currentWrongWayIndex)
        {
            wrongWaySound.Play();
        }
        currentWrongWayIndex = newIndex;
    }
    
    void MapFieldOfView() 
    {
        float angleBetweenRays = angleOfViewInRadian / (numberOfRays - 1);
        
        for (int i = 0; i < numberOfRays; i++) {
            float currentAngle = -angleOfViewInRadian / 2 + angleBetweenRays * i;

            //calculates the direction of the ray
            Vector3 currentDirection =  playerCamera.transform.forward + playerCamera.transform.right * Mathf.Asin(currentAngle);
            

            RaycastHit hit;
            Ray Ray1 = new Ray(playerCamera.transform.position, currentDirection);
            
            
            //Debug.Log("direction: " + currentDirection);
            
            // Cast a ray straight downwards.
            if (Physics.Raycast(Ray1, out hit))
            {
                if (sp.IsOpen) {
                    float distFloat = hit.distance;
                    distFloat *= 10;
                    int distInt = (int)distFloat;

                    Color color;
                    char distChar;
                    if (distInt > 79)
                    {
                        distChar = (char)69;
                        color = new Color(0, 0, 255);
                    } else if (distInt > 59)
                    {
                        distChar = (char)68;
                        color = new Color(0, 125, 125);
                    }
                    else if (distInt > 39)
                    {
                        distChar = (char)67;
                        color = new Color(0, 255, 0);
                    }
                    else if (distInt > 19)
                    {
                        distChar = (char)66;
                        color = new Color(125, 125, 0);
                    } else
                    {
                        distChar = (char)65;
                        color = new Color(255, 0, 0);
                    }

                    sp.Write(distChar.ToString());
                    Debug.DrawLine(playerCamera.transform.position + Vector3.down * 0.2f, playerCamera.transform.position + Vector3.down * 0.2f + currentDirection * 10, color, servoFramRate, true);
                    //Debug.Log("LED " + i + ", distance truncated: " + distInt);
                } else {
                    Debug.Log("Couldn't open connection");
                }
            }
        }
        char startChar = (char)97;
        sp.Write(startChar.ToString());
        //Debug.Log("Ending character:" + startChar);       
    }

    private void Start()
    {        
        characterController = GetComponent<CharacterController>();

        // Lock cursor
        Cursor.lockState = CursorLockMode.Locked;
        Cursor.visible = false;

        
        //Serial initialization
        string the_com="";
        
        /*foreach (string mysps in SerialPort.GetPortNames())
        {
            print(mysps);
            if (mysps != "COM1") { the_com = mysps; break; }
        }*/
        try
        {
            the_com = "/dev/cu.usbserial-1410";
            //sp = new SerialPort("\\\\.\\" + the_com, 9600);
            sp = new SerialPort(the_com, 9600);
            if (!sp.IsOpen)
            {
                print("Opening " + the_com + ", baud 9600");
                sp.ReadTimeout = 500;
                sp.Handshake = Handshake.None;
                sp.Open();
                if (sp.IsOpen) { print("Open"); }
            }

            InvokeRepeating("MapFieldOfView", 0f, servoFramRate);
        } catch (IOException exception)
        {
            Debug.Log("Serial connection not available");
            Debug.Log(exception.StackTrace);
        }

        originalRotation = transform.rotation;
    }

    private void Update()
    {
        // We are grounded, so recalculate move direction based on axes
        Vector3 forward = transform.TransformDirection(Vector3.forward);
        Vector3 right = transform.TransformDirection(Vector3.right);
        // Press Left Shift to run
        bool isRunning = Input.GetKey(KeyCode.LeftShift);
        float curSpeedX = canMove ? (isRunning ? runningSpeed : walkingSpeed) * Input.GetAxis("Vertical") : 0;
        float curSpeedY = canMove ? (isRunning ? runningSpeed : walkingSpeed) * Input.GetAxis("Horizontal") : 0;
        float movementDirectionY = moveDirection.y;
        moveDirection = (forward * curSpeedX) + (right * curSpeedY);

        if (Input.GetButton("Jump") && canMove && characterController.isGrounded)
        {
            moveDirection.y = jumpSpeed;
        }
        else
        {
            moveDirection.y = movementDirectionY;
        }

        // Apply gravity. Gravity is multiplied by deltaTime twice (once here, and once below
        // when the moveDirection is multiplied by deltaTime). This is because gravity should be applied
        // as an acceleration (ms^-2)
        if (!characterController.isGrounded)
        {
            moveDirection.y -= gravity * Time.deltaTime;
        }

        // Move the controller
        characterController.Move(moveDirection * Time.deltaTime);

        // Player and Camera rotation
        if (canMove)
        {
            rotationX += -Input.GetAxis("Mouse Y") * lookSpeed;
            rotationX = Mathf.Clamp(rotationX, -lookXLimit, lookXLimit);
            playerCamera.transform.localRotation = Quaternion.Euler(rotationX, 0, 0);
            transform.rotation *= Quaternion.Euler(0, Input.GetAxis("Mouse X") * lookSpeed, 0);
        }


        //check if player fell down
        if (transform.position.y < -1f && !isFalling)
        {
            GetComponent<AudioSource>().Play();
            isFalling = true;
        }
        if (transform.position.y < -30f)
        {
            isFalling = false;
            moveDirection.y = 0;
            transform.position = Vector3.up * 2;
            transform.rotation = originalRotation;
        }
    }
}